`timescale 1ns / 1ps

//******************************************************************************
//* MiniRISC CPU v2.0                                                          *
//******************************************************************************
module minirisc_cpu(
   //rajel s reset.
   input  wire        clk,             //rajel
   input  wire        rst,             //Aszinkron reset
   
   //Busz interfsz a programmemria elrshez.
   output wire [7:0]  cpu2pmem_addr,   //Cmbusz
   input  wire [15:0] pmem2cpu_data,   //Olvassi adatbusz
   
   //Master busz interfsz az adatmemria elrshez.
   output wire        m_bus_req,       //Busz hozzfrs krse
   input  wire        m_bus_grant,     //Busz hozzfrs megadsa
   output wire [7:0]  m_mst2slv_addr,  //Cmbusz
   output wire        m_mst2slv_wr,    //rs engedlyez jel
   output wire        m_mst2slv_rd,    //Olvass engedlyez jel
   output wire [7:0]  m_mst2slv_data,  //rsi adatbusz
   input  wire [7:0]  m_slv2mst_data,  //Olvassi adatbusz
   
   //Megszaktskr bemenet (aktv magas szintrzkeny).
   input  wire        irq,
   
   //Debug interfsz.
   input  wire [22:0] dbg2cpu_data,    //Jelek a debug modultl a CPU fel
   output wire [47:0] cpu2dbg_data     //Jelek a CPU-tl a debug modul fel
);

//******************************************************************************
//* Debug interfsz jelek.                                                     *
//******************************************************************************
wire [7:0]  dbg_data_in;               //Adat a debug modultl
wire [7:0]  dbg_addr_in;               //Cm a debug modultl
wire        dbg_break;                 //Az utasts vgrehajts felfggesztse
wire        dbg_continue;              //Az utasts vgrehajts folytatsa
wire        dbg_pc_wr;                 //A PC rs engedlyez jele (debug)
wire        dbg_flag_wr;               //A flag-ek rs engedlyez jele
wire        dbg_reg_wr;                //A regisztertmb rs engedlyez jele
wire        dbg_mem_wr;                //Az adatmemria rs engedlyez jele
wire        dbg_mem_rd;                //Az adatmemria olvass engedlyez jele
wire        dbg_instr_dec;             //Az utasts dekdols jelzse
wire        dbg_int_req;               //A megszakts kiszolglsnak jelzse
wire [7:0]  dbg_reg_dout;              //A regisztertmbbl beolvasott adat
wire        dbg_flag_ie;               //Megyszakts engedlyez flag (IE)
wire        dbg_flag_if;               //Megyszakts flag (IF)
wire        dbg_is_brk;                //A trspont llapot jelzse
wire [13:0] dbg_stack_top;             //A verem tetejn lv adat


//******************************************************************************
//* A processzor adatstruktrjt vezrl egysg.                              *
//******************************************************************************
wire       data_mem_wr;                //Adatmemria rs engedlyez jel
wire       data_mem_rd;                //Adatmemria olvass engedlyez jel
wire [7:0] const_data;                 //Az utastsban lv konstans adat
wire       wr_data_sel;                //A regiszterbe rand adat kivlasztsa
wire       addr_op2_sel;               //Az ALU 2. operandusnak kivlasztsa
wire       reg_wr_en;                  //A regisztertmb rs engedlyez jele
wire [3:0] reg_addr_x;                 //Regiszter cme (X port)
wire [3:0] reg_addr_y;                 //Regiszter cme (Y port)
wire [1:0] alu_op_type;                //ALU mvelet kivlaszt jel
wire [1:0] alu_arith_sel;              //Aritmetikai mvelet kivlaszt jel
wire [1:0] alu_logic_sel;              //Logikai mvelet kivlaszt jel
wire [3:0] alu_shift_sel;              //Shiftelsi mvelet kivlaszt jel
wire [3:0] alu_flag_din;               //A flag-ekbe rand rtk
wire       alu_flag_wr;                //A flag-ek rs engedlyez jele
wire       alu_flag_z;                 //Zero flag
wire       alu_flag_c;                 //Carry flag
wire       alu_flag_n;                 //Negative flag
wire       alu_flag_v;                 //Overflow flag
wire [7:0] jump_addr;                  //Ugrsi cm

control_unit control_unit(
   //rajel s reset.
   .clk(clk),                          //rajel
   .rst(rst),                          //Aszinkron reset
   
   //A programmemrival kapcsolatos jelek.
   .prg_mem_addr(cpu2pmem_addr),       //Cmbusz
   .prg_mem_din(pmem2cpu_data),        //Olvassi adatbusz
   
   //Az adatmemrival kapcsolatos jelek.
   .bus_req(m_bus_req),                //Busz hozzfrs krse
   .bus_grant(m_bus_grant),            //Busz hozzfrs megadsa
   .data_mem_wr(data_mem_wr),          //rs engedlyez jel
   .data_mem_rd(data_mem_rd),          //Olvass engedlyez jel
   
   //Az utastsban lv konstans adat.
   .const_data(const_data),
   
   //Az adatstruktra multiplexereinek vezrl jelei.
   .wr_data_sel(wr_data_sel),          //A regiszterbe rand adat kivlasztsa
   .addr_op2_sel(addr_op2_sel),        //Az ALU 2. operandusnak kivlasztsa
   
   //A regisztertmbbel kapcsolatos jelek.
   .reg_wr_en(reg_wr_en),              //rs engedlyez jel
   .reg_addr_x(reg_addr_x),            //Regiszter cme (X port)
   .reg_addr_y(reg_addr_y),            //Regiszter cme (Y port)
   
   //Az ALU-val kapcsolatos jelek.
   .alu_op_type(alu_op_type),          //ALU mvelet kivlaszt jel
   .alu_arith_sel(alu_arith_sel),      //Aritmetikai mvelet kivlaszt jel
   .alu_logic_sel(alu_logic_sel),      //Logikai mvelet kivlaszt jel
   .alu_shift_sel(alu_shift_sel),      //Shiftelsi mvelet kivlaszt jel
   .alu_flag_din(alu_flag_din),        //A flag-ekbe rand rtk
   .alu_flag_wr(alu_flag_wr),          //A flag-ek rs engedlyez jele
   .alu_flag_z(alu_flag_z),            //Zero flag
   .alu_flag_c(alu_flag_c),            //Carry flag
   .alu_flag_n(alu_flag_n),            //Negative flag
   .alu_flag_v(alu_flag_v),            //Overflow flag
   
   //Ugrsi cm az adatstruktrtl.
   .jump_addr(jump_addr),
   
   //Megszaktskr bemenet (aktv magas szintrzkeny).
   .irq(irq),
   
   //A debug interfsz jelei.
   .dbg_data_in(dbg_data_in),          //Adatbemenet
   .dbg_break(dbg_break),              //Program vgrehajtsnak felfggesztse
   .dbg_continue(dbg_continue),        //Program vgrehajtsnak folytatsa
   .dbg_pc_wr(dbg_pc_wr),              //A programszmll rs engedlyez jele
   .dbg_flag_wr(dbg_flag_wr),          //A flag-ek rs engedlyez jele
   .dbg_reg_wr(dbg_reg_wr),            //A regisztertmb rs engedlyez jele
   .dbg_mem_wr(dbg_mem_wr),            //Az adatmemria rs engedlyez jele
   .dbg_mem_rd(dbg_mem_rd),            //Az adatmemria olvass engedlyez jele
   .dbg_instr_dec(dbg_instr_dec),      //Az utasts dekdols jelzse
   .dbg_int_req(dbg_int_req),          //A megszakts kiszolglsnak jelzse
   .dbg_is_brk(dbg_is_brk),            //A trspont llapot jelzse
   .dbg_flag_ie(dbg_flag_ie),          //Megyszakts engedlyez flag (IE)
   .dbg_flag_if(dbg_flag_if),          //Megyszakts flag (IF)
   .dbg_stack_top(dbg_stack_top)       //A verem tetejn lv adat
);


//******************************************************************************
//* A mveleteket vgrehajt adatstruktra.                                    *
//******************************************************************************
wire [7:0] data_mem_addr;
wire [7:0] data_mem_dout;

datapath datapath(
   //rajel.
   .clk(clk),
   
   //Az adatmemrival kapcsolatos jelek.
   .data_mem_addr(data_mem_addr),      //Cmbusz
   .data_mem_din(m_slv2mst_data),      //Olvassi adatbusz
   .data_mem_dout(data_mem_dout),      //rsi adatbusz
   
   //Az utastsban lv konstans adat.
   .const_data(const_data),
   
   //A multiplexerek vezrl jelei.
   .wr_data_sel(wr_data_sel),          //A regiszterbe rand adat kivlasztsa
   .addr_op2_sel(addr_op2_sel),        //Az ALU 2. operandusnak kivlasztsa
   
   //A regisztertmbbel kapcsolatos jelek.
   .reg_addr_x(reg_addr_x),            //Regiszter cme (X port)
   .reg_addr_y(reg_addr_y),            //Regiszter cme (Y port)
   .reg_wr_en(reg_wr_en),              //rs engedlyez jel
   
   //Az ALU-val kapcsolatos jelek.
   .alu_op_type(alu_op_type),          //ALU mvelet kivlaszt jel
   .alu_arith_sel(alu_arith_sel),      //Aritmetikai mvelet kivlaszt jel
   .alu_logic_sel(alu_logic_sel),      //Logikai mvelet kivlaszt jel
   .alu_shift_sel(alu_shift_sel),      //Shiftelsi mvelet kivlaszt jel
   .alu_flag_din(alu_flag_din),        //A flag-ekbe rand rtk
   .alu_flag_wr(alu_flag_wr),          //A flag-ek rs engedlyez jele
   .alu_flag_z(alu_flag_z),            //Zero flag
   .alu_flag_c(alu_flag_c),            //Carry flag
   .alu_flag_n(alu_flag_n),            //Negative flag
   .alu_flag_v(alu_flag_v),            //Overflow flag
   
   //A programszmll j rtke ugrs esetn.
   .jump_address(jump_addr),
   
   //A debug interfsz jelei.
   .dbg_addr_in(dbg_addr_in),          //Cm bemenet
   .dbg_data_in(dbg_data_in),          //Adatbemenet
   .dbg_is_brk(dbg_is_brk),            //A trspont llapot jelzse
   .dbg_reg_dout(dbg_reg_dout)         //A regisztertmbbl beolvasott adat
);


//******************************************************************************
//* Az adatmemria interfsz kimeneteinek meghajtsa. Ha a processzor nem kap  *
//* busz hozzfrst, akkor ezek rtke inaktv nulla kell, hogy legyen.       *
//******************************************************************************
assign m_mst2slv_addr = (m_bus_grant) ? data_mem_addr : 8'd0;
assign m_mst2slv_wr   = (m_bus_grant) ? data_mem_wr   : 1'b0;
assign m_mst2slv_rd   = (m_bus_grant) ? data_mem_rd   : 1'b0;
assign m_mst2slv_data = (m_bus_grant) ? data_mem_dout : 8'd0;


//******************************************************************************
//* A debug interfsz jeleinek meghajtsa.                                     *
//******************************************************************************
assign dbg_data_in         = dbg2cpu_data[7:0];
assign dbg_addr_in         = dbg2cpu_data[15:8];
assign dbg_break           = dbg2cpu_data[16];
assign dbg_continue        = dbg2cpu_data[17];
assign dbg_pc_wr           = dbg2cpu_data[18];
assign dbg_flag_wr         = dbg2cpu_data[19];
assign dbg_reg_wr          = dbg2cpu_data[20];
assign dbg_mem_wr          = dbg2cpu_data[21];
assign dbg_mem_rd          = dbg2cpu_data[22];

assign cpu2dbg_data[7:0]   = cpu2pmem_addr;
assign cpu2dbg_data[15:8]  = dbg_reg_dout;
assign cpu2dbg_data[23:16] = m_slv2mst_data;
assign cpu2dbg_data[37:24] = dbg_stack_top;
assign cpu2dbg_data[38]    = alu_flag_z;
assign cpu2dbg_data[39]    = alu_flag_c;
assign cpu2dbg_data[40]    = alu_flag_n;
assign cpu2dbg_data[41]    = alu_flag_v;
assign cpu2dbg_data[42]    = dbg_flag_ie;
assign cpu2dbg_data[43]    = dbg_flag_if;
assign cpu2dbg_data[44]    = dbg_is_brk;
assign cpu2dbg_data[45]    = m_bus_grant;
assign cpu2dbg_data[46]    = dbg_instr_dec;
assign cpu2dbg_data[47]    = dbg_int_req;

endmodule
