`timescale 1ns / 1ps

//******************************************************************************
//* MiniRISC CPU v2.0                                                          *
//*                                                                            *
//* 16 sz mlysg HW verem a programszmll s az ALU flag-ek elmentshez  *
//* szubrutinhvs, illetve megszaktskrs esetn. A megvalsts elosztott  *
//* memrit hasznl.                                                          *
//******************************************************************************
module stack #(
   //Az adat szlessge bitekben.
   parameter DATA_WIDTH = 8
) (
   //rajel.
   input  wire                  clk,
   
   //Adatvonalak.
   input  wire [DATA_WIDTH-1:0] data_in,     //A verembe rand adat
   output wire [DATA_WIDTH-1:0] data_out,    //A verem tetejn lv adat
   
   //Vezrl bemenetek.
   input  wire                  push,        //Adat rsa a verembe
   input  wire                  pop          //Adat olvassa a verembl
);

//******************************************************************************
//* rsi cmszmll. PUSH mvelet esetn az rtkt nveljk, POP mvelet    *
//* esetn az rtkt cskkentjk.                                             *
//******************************************************************************
reg [3:0] wr_address;

always @(posedge clk)
begin
   if (push)
      wr_address <= wr_address + 4'd1;
   else
      if (pop)
         wr_address <= wr_address - 4'd1;
end


//******************************************************************************
//* Az rsi cmbl egyet kivonva megkapjuk az olvassi cmet.                 *
//******************************************************************************
wire [3:0] rd_address = wr_address - 4'd1;


//******************************************************************************
//* A 16 x DATA_WIDTH bites elosztott RAM. PUSH mvelet esetn berjuk az      *
//* adatot a memriba.                                                        *
//******************************************************************************
(* ram_style = "distributed" *)
reg [DATA_WIDTH-1:0] stack_ram [15:0];

always @(posedge clk)
begin
   if (push)
      stack_ram[wr_address] <= data_in;
end

assign data_out = stack_ram[rd_address];


endmodule
