`timescale 1ns / 1ps

//******************************************************************************
//* Egyszer busz arbiter 2 master egysghez.                                  *
//*                                                                            *
//* A busz hozzfrs megadsa fix priorits temezs szerint trtnik,       *
//* az MST0 rendelkezik a nagyobb prioritssal. Tbb master egysg egyidej    *
//* krse esetn a nagyobb priorits kapja meg a buszt. A busz hozzfrssel *
//* nem rendelkez master egysgeknek inaktv nulla rtkkel kell meghajtaniuk *
//* a kimeneteiket.                                                            *
//*                                                                            *
//*                   ------        ------        ------        ------         *
//* CLK              |      |      |      |      |      |      |      |        *
//*               ---        ------        ------        ------        ---     *
//*                                                                            *
//* Priorits           MASTER 0   |   MASTER 0  |   MASTER 0  |  MASTER 0     *
//*                                                                            *
//*                                  -------------                             *
//* MST0_REQ                        /             \                            *
//*               ------------------               -----------------------     *
//*                   ------------------------------------------               *
//* MST1_REQ         /                                          \              *
//*               ---                                            ---------     *
//*                                   --------------                           *
//* MST0_GRANT                       /              \                          *
//*               -------------------                ---------------------     *
//*                    --------------                -------------             *
//* MST1_GRANT        /              \              /             \            *
//*               ----                --------------               -------     *
//*                                                                            *
//*                    -------------- -------------- --------------            *
//* MST2SLV_WR        /  MST1 RD/WR  X  MST0 RD/WR  X  MST1 RD/WR  \           *
//* MST2SLV_RD    ------------------- -------------- --------------------      *
//*               ---- -------------- -------------- -------------- -----      *
//* MST2SLV_ADDR   0  X   MST1 CM   X   MST0 CM   X   MST1 CM   X   0       *
//*               ---- -------------- ---- ------------------------ -----      *
//*               ---- -------------- -------------- -------------- -----      *
//* MST2SLV_DATA   0  X   MST1 ADAT  X   MST0 ADAT  X   MST1 ADAT  X   0       *
//* SLV2MST_DATA  ---- -------------- -------------- -------------- -----      *
//*                                 ^              ^              ^            *
//*                    Az rsi vagy olvassi parancsok itt hajtdnak vgre.   *
//******************************************************************************
module bus_arbiter_2m_fixed(
   //A master 0 egysghez tartoz jelek.
   input  wire mst0_req,         //Busz hozzfrs krse
   output wire mst0_grant,       //Busz hozzfrs megadsa
   
   //A master 1 egysghez tartoz jelek.
   input  wire mst1_req,         //Busz hozzfrs krse
   output wire mst1_grant        //Busz hozzfrs megadsa
);

//******************************************************************************
//* Priorits dekder.                                                         *
//******************************************************************************
//A priorits dekder kimenete.
reg [1:0] grant;

//A priorits dekder bemenete.
wire [2:0] decoder_in = {mst1_req, mst0_req};

always @(*)                       //a buszt kri    prioritsa      a buszt
begin                             //MST0    MST1        van         megkapja
   case (decoder_in)              //------------------------------------------
      2'b00: grant <= 2'b00;      //nem     nem      MST0-nak   ->  egyik sem
      2'b01: grant <= 2'b01;      //igen    nem      MST0-nak   ->    MST0 
      2'b10: grant <= 2'b10;      //nem     igen     MST0-nak   ->    MST1 
      2'b11: grant <= 2'b01;      //igen    igen     MST0-nak   ->    MST0
   endcase
end

//A busz hozzfrs megads jelek meghajtsa.
assign mst0_grant = grant[0];
assign mst1_grant = grant[1];

endmodule
