`timescale 1ns / 1ps

//******************************************************************************
//* Egyszer busz arbiter 2 master egysghez.                                  *
//*                                                                            *
//* A busz hozzfrs megadsa round-robin (krforgsos) temezs szerint      *
//* trtnik, azaz minden hozzfrs utn a kvetkez master egysgnek lesz    *
//* nagyobb priortsa. Tbb master egysg egyidej krse esetn a nagyobb    *
//* priorts kapja meg a buszt. A busz hozzfrssel nem rendelkez master   *
//* egysgeknek inaktv nulla rtkkel kell meghajtaniuk a kimeneteiket.       *
//*                                                                            *
//*                   ------        ------        ------        ------         *
//* CLK              |      |      |      |      |      |      |      |        *
//*               ---        ------        ------        ------        ---     *
//*                                                                            *
//* Priorits           MASTER 0   |   MASTER 1  |   MASTER 0  |  MASTER 1     *
//*                                                                            *
//*                   ------------------------------------------               *
//* MST0_REQ         /                                          \              *
//*               ---                                            ---------     *
//*                                  -------------                             *
//* MST1_REQ                        /             \                            *
//*               ------------------               -----------------------     *
//*                    --------------                -------------             *
//* MST0_GRANT        /              \              /             \            *
//*               ----                --------------               -------     *
//*                                   --------------                           *
//* MST1_GRANT                       /              \                          *
//*               -------------------                ---------------------     *
//*                                                                            *
//*                    -------------- -------------- --------------            *
//* MST2SLV_WR        /  MST0 RD/WR  X  MST1 RD/WR  X  MST0 RD/WR  \           *
//* MST2SLV_RD    ------------------- -------------- --------------------      *
//*               ---- -------------- -------------- -------------- -----      *
//* MST2SLV_ADDR   0  X   MST0 CM   X   MST1 CM   X   MST0 CM   X   0       *
//*               ---- -------------- ---- ------------------------ -----      *
//*               ---- -------------- -------------- -------------- -----      *
//* MST2SLV_DATA   0  X   MST0 ADAT  X   MST1 ADAT  X   MST0 ADAT  X   0       *
//* SLV2MST_DATA  ---- -------------- -------------- -------------- -----      *
//*                                 ^              ^              ^            *
//*                    Az rsi vagy olvassi parancsok itt hajtdnak vgre.   *
//******************************************************************************
module bus_arbiter_2m_rr(
   //rajel s reset.
   input  wire clk,              //rajel
   input  wire rst,              //Reset jel
   
   //A master 0 egysghez tartoz jelek.
   input  wire mst0_req,         //Busz hozzfrs krse
   output wire mst0_grant,       //Busz hozzfrs megadsa
   
   //A master 1 egysghez tartoz jelek.
   input  wire mst1_req,         //Busz hozzfrs krse
   output wire mst1_grant        //Busz hozzfrs megadsa
);

//******************************************************************************
//* Priorits dekder.                                                         *
//******************************************************************************
//A prioritst kivlaszt jel:
//0: a master 0 egysgnek van nagyobb prioritsa
//1: a master 1 egysgnek van nagyobb prioritsa
reg priority_sel;

//A priorits dekder kimenete.
reg [1:0] grant;

//A priorits dekder bemenete.
wire [2:0] decoder_in = {priority_sel, mst1_req, mst0_req};

always @(*)                        //a buszt kri    prioritsa      a buszt
begin                              //MST0    MST1        van         megkapja
   case (decoder_in)               //------------------------------------------
      3'b000: grant <= 2'b00;      //nem     nem      MST0-nak   ->  egyik sem
      3'b001: grant <= 2'b01;      //igen    nem      MST0-nak   ->    MST0 
      3'b010: grant <= 2'b10;      //nem     igen     MST0-nak   ->    MST1 
      3'b011: grant <= 2'b01;      //igen    igen     MST0-nak   ->    MST0
      3'b100: grant <= 2'b00;      //nem     nem      MST1-nek   ->  egyik sem      
      3'b101: grant <= 2'b01;      //igen    nem      MST1-nek   ->    MST0 
      3'b110: grant <= 2'b10;      //nem     igen     MST1-nek   ->    MST1 
      3'b111: grant <= 2'b10;      //igen    igen     MST1-nek   ->    MST1 
   endcase
end

//A busz hozzfrs megads jelek meghajtsa.
assign mst0_grant = grant[0];
assign mst1_grant = grant[1];


//******************************************************************************
//* A round-robin (krforgsos) temezs megvalstsa: minden busz hozzfrs *
//* utn a kvetkez master egysgnek lesz nagyobb prioritsa.                 *
//******************************************************************************
always @(posedge clk)
begin
   if (rst)
      priority_sel <= 1'b0;
   else
      if (grant != 0)
         priority_sel <= ~priority_sel;
end

endmodule
