`timescale 1ns / 1ps

//******************************************************************************
//* Alap perifria a htszegmenses s a pontmtrix kijelz illesztshez.      *
//* Az egyes szegmensek egyedileg vezrelhetk, a kijelzk idmultiplexelt     *
//* vezrlst a perifria elvgzi.                                            *
//*                                                                            *
//* A perifria cme paramter tadssal llthat be a fels szint modulban  *
//* a megpldnyostskor. A szintzis mr ennek megfeleleen az aktulis      *
//* BASEADDR bziscmmel trtnik. A perifria 16 bjtos cmtartomnyt ignyel,*
//* ebbl csak az als 9 bjt van felhasznlva.                                *
//*                                                                            *
//* A programozi fellet:                                                     *
//*                                                                            *
//* Cm         Tpus   Bitek                                                  *
//* BASEADDR+0  R/W     DIG0 adatregiszter                                     *
//* BASEADDR+1  R/W     DIG1 adatregiszter                                     *
//* BASEADDR+2  R/W     DIG2 adatregiszter                                     *
//* BASEADDR+3  R/W     DIG3 adatregiszter                                     *
//*                     DP    G     F     E     D     C     B     A            *
//* BASEADDR+4  R/W     COL0 adatregiszter                                     *
//* BASEADDR+5  R/W     COL1 adatregiszter                                     *
//* BASEADDR+6  R/W     COL2 adatregiszter                                     *
//* BASEADDR+7  R/W     COL3 adatregiszter                                     *
//* BASEADDR+8  R/W     COL4 adatregiszter                                     *
//*                     -     ROW7  ROW6  ROW5  ROW4  ROW3  ROW2  ROW1         *
//******************************************************************************
module basic_display #(
   //A perifria bziscme.
   parameter BASEADDR = 8'hff
) (
   //rajel s reset.
   input  wire       clk,              //rajel
   input  wire       rst,              //Reset jel
   
   //A slave busz interfsz jelei.
   input  wire [7:0] s_mst2slv_addr,   //Cmbusz
   input  wire       s_mst2slv_wr,     //rs engedlyez jel
   input  wire       s_mst2slv_rd,     //Olvass engedlyez jel
   input  wire [7:0] s_mst2slv_data,   //rsi adatbusz
   output wire [7:0] s_slv2mst_data,   //Olvassi adatbusz
   
   //A kijelzk vezrlshez szksges jelek.
   output wire [7:0] seg_n,            //Szegmens vezrl jelek (aktv alacsony)
   output wire [3:0] dig_n,            //Digit kivlaszt jelek (aktv alacsony)
   output wire [4:0] col_n             //Oszlop kivlaszt jelek (aktv alacsony)
);

//******************************************************************************
//* Cmdekdols.                                                              *
//******************************************************************************
//A perifria kivlaszt jele.
wire psel = ((s_mst2slv_addr >> 4) == (BASEADDR >> 4));


//******************************************************************************
//* Szmll a memria trlshez.                                             *
//******************************************************************************
reg  [4:0] rst_cnt;
wire       mem_clr = (rst_cnt[4] == 0);

always @(posedge clk)
begin
   if (rst)
      rst_cnt <= 5'd0;
   else
      if (mem_clr)
         rst_cnt <= rst_cnt + 5'd1;
end


//******************************************************************************
//* A regisztereket egy 16 x 8 bites elosztott RAM-mal valstjuk meg.         *
//******************************************************************************
(* ram_style = "distributed" *)
reg  [7:0] registers [15:0];
wire [3:0] reg_addr = (mem_clr) ? rst_cnt[3:0] : s_mst2slv_addr[3:0];
wire [7:0] reg_din  = (mem_clr) ? 8'd0         : s_mst2slv_data;
wire       reg_wr   = mem_clr | (psel & s_mst2slv_wr);

always @(posedge clk)
begin
   if (reg_wr)
      registers[reg_addr] <= reg_din;
end

//Az olvassi adatbusz meghajtsa.
assign s_slv2mst_data = (psel & s_mst2slv_rd) ? registers[reg_addr] : 8'd0;


//******************************************************************************
//* A rendszerrajel leosztsa 16385-tel (2^14 + 1).                           *
//******************************************************************************
reg  [14:0] clk_divider;
wire        clk_divider_tc = clk_divider[14];

always @(posedge clk)
begin
   if (rst || clk_divider_tc)
      clk_divider <= 15'd0;
   else
      clk_divider <= clk_divider + 15'd1;
end


//******************************************************************************
//* A RAM olvassi cmszmllja.                                              *
//******************************************************************************
reg [3:0] rd_addr;

always @(posedge clk)
begin
   if (rst)
      rd_addr <= 4'd0;
   else
      if (clk_divider_tc)
         if (rd_addr[3])
            rd_addr <= 4'd0;
         else
            rd_addr <= rd_addr + 4'd1;
end

//A szegmens vezrl jelek meghajtsa.
assign seg_n = ~registers[rd_addr];


//******************************************************************************
//* A digit s oszlop kivlaszt jeleket elllt shiftregiszter.             *
//******************************************************************************
reg [8:0] shr;

always @(posedge clk)
begin
   if (rst)
      shr <= 9'b00000_0001;
   else
      if (clk_divider_tc)
         shr <= {shr[7:0], shr[8]};
end

//A digit s oszlop kivlaszt jelek meghajtsa.
assign dig_n = ~shr[3:0];
assign col_n = ~shr[8:4];


endmodule
