`timescale 1ns / 1ps

//******************************************************************************
//* Egyszer 8 bites bemeneti modul. Semmi extra szolgltats, csak            * 
//* mintavtelezs trtnik.                                                   *
//*                                                                            *
//* A perifria cme paramter tadssal llthat be a fels szint modulban. *
//* a GPIO modul megpldnyostsakor. A szintzis mr ennek megfeleleen az   *
//* aktulis BASEADDR bziscmmel trtnik.                                    *
//*                                                                            *
//* A processzor olvassi (READ) ciklusnak iddiagramja:                      *
//* Az olvassi ciklust az 1 rajel ciklus ideig aktv S_MST2SLV_RD jel jelzi. *
//* Az olvassi ciklus ideje alatt a cm stabil s a kivlasztott perifria az *
//* olvassi adatbuszra kapuzza az adatot. Az olvassi ciklus eltt s utn    *
//* az olvassi adatbusz rtke inaktv 0 kell, hogy legyen.                   *
//*                                                                            *
//*                     --------          --------          --------           *
//* CLK                |        |        |        |        |        |          *
//*                ----          --------          --------          ----      *
//*                                        ------------------                  *
//* S_MST2SLV_RD                          /                  \                 *
//*                -----------------------                    -----------      *
//*                ----------------------- ------------------ -----------      *
//* S_MST2SLV_ADDR                        X   RVNYES CM   X                 *
//*                ----------------------- ------------------ -----------      *
//*                ----------------------- ------------------ -----------      *
//* S_SLV2MST_DATA           0            X   RVNYES ADAT  X       0         *
//*                ----------------------- ------------------ -----------      *
//*                                                        ^                   *
//*                              A bemeneti adat mintavtelezse itt trtnik. *  
//******************************************************************************
module basic_in #(
   //A perifria bziscme.
   parameter BASEADDR = 8'hff
) (
   //rajel s reset.
   input  wire       clk,              //rajel
   input  wire       rst,              //Reset jel
   
   //A slave busz interfsz jelei.
   input  wire [7:0] s_mst2slv_addr,   //Cmbusz
   input  wire       s_mst2slv_rd,     //Olvass engedlyez jel
   output wire [7:0] s_slv2mst_data,   //Olvassi adatbusz
   
   //A GPIO interfsz jelei.
   input  wire [7:0] gpio_in           //Az IO lbak aktulis rtke
);

//******************************************************************************
//* Cmdekdols.                                                              *
//******************************************************************************
//A perifria kivlaszt jele.
wire psel = (s_mst2slv_addr == BASEADDR);

//A bemeneti adatregiszter olvassnak jelzse.
wire in_reg_rd = psel & s_mst2slv_rd;


//******************************************************************************
//* A bemeneti adatregiszter.                                                  *
//*                                                                            *
//* Az alapfunkcit egyszer minavtelezssel valstjuk meg, melyet minden    * 
//* rajel ciklusban elvgznk.                                                *
//******************************************************************************
reg [7:0] in_reg;

always @(posedge clk)
begin
   if (rst)
      in_reg <= 8'd0;                  //Reset esetn trljk a regisztert
   else
      in_reg <= gpio_in;               //Egybknt folyamatosan mintavtelezzk
end                                    //az IO lbak rtkt


//******************************************************************************
//* A processzor olvassi adatbusznak meghajtsa. Az olvassi adatbuszra csak *
//* az olvass ideje alatt kapcsoljuk r a krt rtket, egybknt egy inaktv *
//* nulla rtk jelenik meg rajta (elosztott busz multiplexer funkci).        *
//******************************************************************************
assign s_slv2mst_data = (in_reg_rd) ? in_reg : 8'd0;

endmodule
