`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Egyszer 8 bites kimeneti modul a PicoBlaze vezrlhz
// Semmi extra szolgltats, csak kzvetlen adat kiads
// A perifria cm paramter tadssal llthat be a fels szint modulban
// a szintzis mr ennek megfeleleen az aktulis p_addr cmmel trtnik

// PicoBlaze IO port WRITE ciklus iddiagram
// A port cm s a kimeneti adat 2 rajel cikluson keresztl stabil
// A WRITE_STROBE a 2. rajelben aktv 
// Ennek megfelelen a kimeneti adatot a 2. rajel vgn rjuk be a kimeneti regiszterbe

// CLK      ________--------________--------________--------________--------______   
// PORT_ID  XXXXXXXXXX<        WRITE_PORT_ID         >XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// DATA_INPUT XXXXXXXX<        VALIID OUTPUT         >XXXXXXXXXXXXXXXXXXXXXXXXXXXXX     
// WRITE_STROBE_______________________----------------______________________________
// Kimenet trsa                                   ^ <- itt

  
module Basic_OUT(
    input clk,
    input rst,
    input wr_strobe,
    input [7:0] addr,
    input [7:0] out_data,                    // A vezrl kimeneti adata ezen a modulon bemenet
    output [7:0] out_signals
    );

parameter p_addr = 8'hff;                    // A cm alaprtke legyen hexa FF

wire p_sel;                                  // Perifria cm kivlaszts jel
wire p_write;                                // Perifria rs jel
reg [7:0] out_reg;                           // Kimeneti regiszter 

// Az alapfunkcit egyszer adatkiadssal valstjuk meg
// A kimeneti adat egy regiszterbe kerl, ez fogja meghajtani a kimeneti vonalakat
// Teht a kimeneti buszon egy rsi ciklus ideig (2 rajel ciklus) megjelen dinamikus 
// adatot a regiszter egy statikus, stabil adatt alaktja (amg jra nem rjuk)

assign p_sel = (addr == p_addr);             // A perifria cm dekdolsa

assign p_write = p_sel & wr_strobe;          // Az rs parancs felismerse

always @ (posedge clk)                      
   if (rst)        out_reg <= 8'b0;          // RESET alatt a regiszter trlve
   else 
      if (p_write) out_reg <= out_data;      // rsnl frisstve, egybknt tartja az llapott


assign out_signals = out_reg;                // A regiszter tartalma kzvetlenl megjelenik a kimeneten


endmodule