`timescale 1ns / 1ps

//******************************************************************************
//* PS/2 vev modul.                                                           *
//******************************************************************************
module ps2_receiver(
   //rajel s reset.
   input  wire       clk,              //rajel
   input  wire       rst,              //Reset jel
   
   //A PS/2 interfsz jelei.
   input  wire       ps2_clk,          //rajel bemenet
   input  wire       ps2_data,         //Soros adatbemenet
   
   //A vtelt engedlyez jel.
   input  wire       rx_enable,
   
   //Prhuzamos adatkimenet.
   output reg  [7:0] data_out,         //A vett adat
   output reg        data_valid        //rvnyes adat jelzse
);

//******************************************************************************
//* Az 1 MHz-es engedlyez jel ellltsa a PS/2 jelek mintavtelezshez.   *
//******************************************************************************
reg  [3:0] clk_divider;
wire       clk_divider_tc = (clk_divider == 4'd15);

always @(posedge clk)
begin
   clk_divider <= clk_divider + 4'd1;
end


//******************************************************************************
//* A PS/2 interfsz jelek mintavtelezse s ldetektls az rajelen.        *
//******************************************************************************
reg  [1:0] ps2_data_samples;
reg  [2:0] ps2_clk_samples;
wire       ps2_clk_falling = (ps2_clk_samples[2:1] == 2'b10) & clk_divider_tc;

always @(posedge clk)
begin
   if (clk_divider_tc)
      ps2_clk_samples <= {ps2_clk_samples[1:0], ps2_clk};
end

always @(posedge clk)
begin
   if (clk_divider_tc)
      ps2_data_samples <= {ps2_data_samples[0], ps2_data};
end


//******************************************************************************
//* Shiftregiszter a soros/prhuzamos talaktshoz. A PS/2 keret felptse:  *
//* START bit (0) | 8 adatbit | Pritsbit (pratlan) | STOP bit (1)            *
//******************************************************************************
reg  [10:0] rx_shr;
//A shiftregiszter alapllapotba llt jele:
// - rendszer reset vagy
// - a vtel nincs engedlyezve vagy
// - a START bit 0
wire        rx_shr_rst = rst | ~rx_enable | ~rx_shr[0];

//A vteli shiftregiszter. A lptets a PS/2 rajel lefut lre trtnik.
always @(posedge clk)
begin
   if (rx_shr_rst)
      rx_shr <= 11'b111_1111_1111;
   else
      if (ps2_clk_falling)
         rx_shr <= {ps2_data_samples[1], rx_shr[10:1]};
end

//A prhuzamos adatkimenet meghajtsa.
always @(posedge clk)
begin
   data_out <= rx_shr[8:1];
end

//Az rvnyes adat jelzse:
// - a START bit 0 s
// - rvnyes pratlan parits s
// - a STOP bit 1
always @(posedge clk)
begin
   if (rst)
      data_valid <= 1'b0;
   else
      data_valid <= ~rx_shr[0] & ^rx_shr[9:1] & rx_shr[10];
end


endmodule
